Rem
Rem $Header: rdbms/admin/noncdb_to_pdb.sql /st_rdbms_12.1.0.2.0dbpsu/1 2014/10/24 09:31:13 sankejai Exp $
Rem
Rem noncdb_to_pdb.sql
Rem
Rem Copyright (c) 2011, 2014, Oracle and/or its affiliates. 
Rem All rights reserved.
Rem
Rem    NAME
Rem      noncdb_to_pdb.sql - Convert PDB
Rem
Rem    DESCRIPTION
Rem      Converts DB to PDB.
Rem
Rem    NOTES
Rem      Given a DB with proper obj$ common bits set, we convert it to a proper
Rem      PDB by deleting unnecessary metadata.
Rem
Rem    MODIFIED   (MM/DD/YY)
Rem    apfwkr      10/09/14 - Backport sankejai_bug-19001359 from main
Rem    huiz        06/06/14 - 18918217: only make type w/o annotated name local 
Rem    apfwkr      06/05/14 - Backport pyam_bug-18727940 from main
Rem    apfwkr      05/19/14 - Backport huiz_localtype from main
Rem    pyam        05/12/14 - Backport pyam_bug-18478064 from main
Rem    sankejai    08/13/14 - 19001359: compare type# when marking users/roles
Rem                           as common
Rem    huiz        06/06/14 - 18918217: only make type w/o annotated name local
Rem    pyam        05/28/14 - 18727940: before marking type as common, confirm
Rem                           that signature and hashcode match
Rem    pyam        04/17/14 - 18063022,18595341: call utlip, flush shared pool
Rem                           before alter table upgrade
Rem    apfwkr      04/15/14 - Backport huiz_bug-18406799 from main
Rem    huiz        05/07/14 - bug 18692867: mark nested tables created via 
Rem    pyam        04/08/14 - 18478064: call reenable_indexes.sql
Rem    huiz        04/03/14 - bug 18406799: mark default tables created via 
Rem                           xml schema registration as local 
Rem    pyam        04/08/14 - 18478064: call reenable_indexes.sql
Rem    jaeblee     04/03/14 - set serveroutput off for later portions of script
Rem    apfwkr      03/17/14 - Backport pyam_bug-17630050 from main
Rem    jaeblee     03/14/14 - pdb should be in restricted mode, not upgrade mode,
Rem                           during utlrp
Rem    jaeblee     03/14/14 - lrg 11577773/11285945: set nls_length_semantics
Rem                           explicitly
Rem    pyam        03/05/14 - convert local to common privileges
Rem    jaeblee     03/05/14 - pdb should be in restricted mode, not upgrade
Rem                           mode, during utlrp
Rem    pyam        01/27/14 - 18043599: add enabled$indexes
Rem    huiz        01/14/14 - bug 18056347: mark types created via xml schema 
Rem                           registration as local 
Rem    mjungerm    01/11/14 - normalize dependencies for all MDL java objects,
Rem                           not just classes.bin - bug 1731977
Rem    pyam        12/19/13 - 17976551: fix ALTER TABLE UPGRADE query
Rem    pyam        12/13/13 - ALTER TABLE UPGRADE bef clearing "_ORACLE_SCRIPT"
Rem    pyam        10/29/13 - move ALTER TABLE UPGRADE of type deps down
Rem    gravipat    10/10/13 - 17562904: Open pdb in upgrade mode until
Rem                           noncdb_to_pdb is run successfully
Rem    pyam        10/03/13 - 17490498: remove pdb bounce, remove
Rem                           _noncdb_to_pdb
Rem    thbaby      08/29/13 - 14515351: add INT$ views for sharing=object
Rem    talliu      08/07/13 - change to close immediate
Rem    pyam        07/26/13 - fix missing &pdbid
Rem    pyam        06/25/13 - 16935643: alter table upgrade for type deps
Rem    talliu      06/24/13 - 16967214: add instances = all for close
Rem    sankejai    04/11/13 - 16530655: do not update status in container$
Rem    pyam        04/03/13 - rename temp cdb$* views, to not interfere when
Rem                           this is run in multiple PDBs simultaneously
Rem    pyam        02/06/13 - error out for non-CDB
Rem    pyam        01/21/13 - stop exiting on sqlerror at end
Rem    mjungerm    01/17/13 - XbranchMerge mjungerm_bug-16171710 from
Rem                           st_rdbms_12.1.0.1
Rem    pyam        01/15/13 - leave PDB in state in which it started
Rem    pyam        11/30/12 - delete services SYS$BACKGROUND and SYS$USERS
Rem    jomcdon     11/16/12 - bug 15894059: fix DBRM code for non-SYS
Rem    pyam        11/15/12 - set nls_length_semantics=byte
Rem    pyam        11/13/12 - skip old version types, update common user bit
Rem    jomcdon     11/07/12 - bug 14800566: fix resource manager plans for pdb
Rem    pyam        10/18/12 - add _noncdb_to_pdb
Rem    pyam        09/27/12 - support plug of upgraded db
Rem    pyam        08/13/12 - remove switches from PL/SQL, disable system 
Rem                           triggers
Rem    pyam        06/26/12 - rename script to noncdb_to_pdb.sql
Rem    pyam        02/23/12 - validate v_$parameter properly
Rem    pyam        09/29/11 - Created
Rem

SET ECHO ON
SET SERVEROUTPUT ON
SET FEEDBACK 1
SET NUMWIDTH 10
SET LINESIZE 80
SET TRIMSPOOL ON
SET TAB OFF
SET PAGESIZE 100

WHENEVER SQLERROR EXIT;

DOC
#######################################################################
#######################################################################
   The following statement will cause an "ORA-01403: no data found"
   error if we're not in a PDB.
   This script is intended to be run right after plugin of a PDB,
   while inside the PDB.
#######################################################################
#######################################################################
#

VARIABLE cdbname VARCHAR2(128)
VARIABLE pdbname VARCHAR2(128)
BEGIN
  SELECT sys_context('USERENV', 'CDB_NAME') 
    INTO :cdbname
    FROM dual
    WHERE sys_context('USERENV', 'CDB_NAME') is not null;
  SELECT sys_context('USERENV', 'CON_NAME') 
    INTO :pdbname
    FROM dual
    WHERE sys_context('USERENV', 'CON_NAME') <> 'CDB$ROOT';
END;
/

COLUMN pdbname NEW_VALUE pdbname
COLUMN pdbid NEW_VALUE pdbid

select :pdbname pdbname from dual;

select TO_CHAR(con_id) pdbid from v$pdbs where name='&pdbname';

-- save pluggable database open mode
COLUMN open_state_col NEW_VALUE open_sql;
COLUMN restricted_col NEW_VALUE restricted_state;
SELECT decode(open_mode,
              'READ ONLY', 'ALTER PLUGGABLE DATABASE &pdbname OPEN READ ONLY',
              'READ WRITE', 'ALTER PLUGGABLE DATABASE &pdbname OPEN',
              'MIGRATE', 'ALTER PLUGGABLE DATABASE &pdbname OPEN UPGRADE', '')
         open_state_col,
       decode(restricted, 'YES', 'RESTRICTED', '')
         restricted_col
       from v$pdbs where name='&pdbname';

-- if pdb was already closed, don't exit on error
WHENEVER SQLERROR CONTINUE;

alter pluggable database "&pdbname" close IMMEDIATE instances = all;

WHENEVER SQLERROR EXIT;

alter session set container = CDB$ROOT;
alter system flush shared_pool;
/
/
alter session set container = "&pdbname";

alter pluggable database "&pdbname" open upgrade;

-- initial setup before beginning the script
alter session set "_ORACLE_SCRIPT"=true;
alter session set NLS_LENGTH_SEMANTICS=BYTE; 
exec dbms_pdb.noncdb_to_pdb(1);

-- if we're plugging in a database that had been upgraded, we need to:
-- 1) generate signatures for common tables
-- 2) patch up tables that with column order differences. This can happen due
--    to db creation scripts adding columns to the middle of a table vs
--    upgrade scripts adding to the end via ALTER TABLE ADD 

alter session set container=CDB$ROOT;

-- create temporary object-linked view to get list of objects marked as common
-- in CDB$ROOT
create or replace view sys.int$cdb$common_root_objects&pdbid sharing=object as
select u.name owner, o.name object_name, o.type# object_type, o.namespace nsp,
       o.subname object_subname, o.signature object_sig,
       decode(bitand(o.flags, 65536), 65536, 'MDL', 'OBL') sharing
  from sys.obj$ o, sys.user$ u where
  o.owner#=u.user# and bitand(o.flags, 196608) <> 0;
create or replace view sys.cdb$common_root_objects&pdbid
as
select * from sys.int$cdb$common_root_objects&pdbid;

create or replace view sys.int$cdb$cmn_root_types&pdbid sharing=object as
select u.name owner, o.name object_name, o.type# object_type, o.namespace nsp,
       o.subname object_subname, o.signature object_sig, t.hashcode hashcode,
       decode(bitand(o.flags, 65536), 65536, 'MDL', 'OBL') sharing
  from sys.obj$ o, sys.user$ u, sys.type$ t where
  o.type#=13 and o.oid$=t.tvoid and o.owner#=u.user# and
  bitand(o.flags, 65536) <> 0;
create or replace view sys.cdb$cmn_root_types&pdbid
as
select * from sys.int$cdb$cmn_root_types&pdbid;


-- object-linked view for list of common users
create or replace view sys.int$cdb$common_users&pdbid sharing=object as
select name, type# from sys.user$ where bitand(spare1, 128) <> 0;
create or replace view sys.cdb$common_users&pdbid
as
select * from sys.int$cdb$common_users&pdbid;

-- object-linked view for accessing dependency$
create or replace view sys.int$cdb$rootdeps&pdbid sharing=object as select du.name as owner, do.name as name, do.type# as d_type#, do.namespace as d_namespace,pu.name as referenced_owner, po.name as referenced_name, po.type# as p_type#, po.namespace as p_namespace,d.order#,d.property,d.d_attrs,d.d_reason from sys.obj$ do, sys.obj$ po, sys.user$ du, sys.user$ pu, sys.dependency$ d where du.user#=do.owner# and pu.user#=po.owner# and do.obj#=d_obj# and po.obj#=p_obj#;
create or replace view sys.cdb$rootdeps&pdbid
as
select * from sys.int$cdb$rootdeps&pdbid;

-- object-linked view for accessing sysauth$: common system privileges
create or replace view sys.int$cdb$commonsysprivs&pdbid sharing=object as
select u.name, s.privilege# from sys.user$ u, sys.sysauth$ s
where s.grantee#=u.user# and bitand(s.option$,8)=8 and s.privilege#<0;

create or replace view sys.cdb$commonsysprivs&pdbid as
select * from sys.int$cdb$commonsysprivs&pdbid;

-- object-linked view for accessing sysauth$: common role grants
create or replace view sys.int$cdb$commonrolegrants&pdbid sharing=object as
select u.name, r.name rolename from sys.user$ u, sys.sysauth$ s, sys.user$ r
where s.grantee#=u.user# and bitand(s.option$,8)=8 and s.privilege#>0 and
s.privilege#=r.user#;

create or replace view sys.cdb$commonrolegrants&pdbid as
select * from sys.int$cdb$commonrolegrants&pdbid;

-- object-linked view for accessing objauth$
create or replace view sys.int$cdb$commonobjprivs&pdbid sharing=object as
select u.name, oa.privilege#, u2.name owner, o.name objname, oa.col#
from sys.user$ u, sys.objauth$ oa, sys.obj$ o, sys.user$ u2
where oa.grantee#=u.user# and bitand(oa.option$,8)=8 
and oa.obj#=o.obj# and o.owner#=u2.user#;

create or replace view sys.cdb$commonobjprivs&pdbid as
select * from sys.int$cdb$commonobjprivs&pdbid;

-- switch into PDB
alter session set container="&pdbname";

create or replace view sys.int$cdb$common_root_objects&pdbid sharing=object as
select u.name owner, o.name object_name, o.type# object_type, o.namespace nsp,
       o.subname object_subname, o.signature object_sig,
       decode(bitand(o.flags, 65536), 65536, 'MDL', 'OBL') sharing
  from sys.obj$ o, sys.user$ u where
  o.owner#=u.user# and bitand(o.flags, 196608) <> 0;
create or replace view sys.cdb$common_root_objects&pdbid
as
select * from sys.int$cdb$common_root_objects&pdbid;

create or replace view sys.int$cdb$cmn_root_types&pdbid sharing=object as
select u.name owner, o.name object_name, o.type# object_type, o.namespace nsp,
       o.subname object_subname, o.signature object_sig, t.hashcode hashcode,
       decode(bitand(o.flags, 65536), 65536, 'MDL', 'OBL') sharing
  from sys.obj$ o, sys.user$ u, sys.type$ t where
  o.type#=13 and o.oid$=t.tvoid and o.owner#=u.user# and
  bitand(o.flags, 65536) <> 0;
create or replace view sys.cdb$cmn_root_types&pdbid
as
select * from sys.int$cdb$cmn_root_types&pdbid;

create or replace view sys.int$cdb$common_users&pdbid sharing=object as
select name, type# from sys.user$ where bitand(spare1, 128) <> 0;
create or replace view sys.cdb$common_users&pdbid
as
select * from sys.int$cdb$common_users&pdbid;

create or replace view sys.int$cdb$rootdeps&pdbid sharing=object as select du.name as owner, do.name as name, do.type# as d_type#, do.namespace as d_namespace,pu.name as referenced_owner, po.name as referenced_name, po.type# as p_type#, po.namespace as p_namespace,d.order#,d.property,d.d_attrs,d.d_reason from sys.obj$ do, sys.obj$ po, sys.user$ du, sys.user$ pu, sys.dependency$ d where du.user#=do.owner# and pu.user#=po.owner# and do.obj#=d_obj# and po.obj#=p_obj#;
create or replace view sys.cdb$rootdeps&pdbid
as
select * from sys.int$cdb$rootdeps&pdbid;

create or replace view sys.cdb$objects&pdbid sharing=none as
select u.name owner, o.name object_name, o.signature object_sig, o.namespace nsp,
       o.subname object_subname, o.obj# object_id, o.type# object_type, o.flags flags
  from sys.obj$ o, sys.user$ u
  where o.owner#=u.user#;

create or replace view sys.cdb$types&pdbid sharing=none as
select u.name owner, o.name object_name, o.signature object_sig, o.namespace nsp,
       o.subname object_subname, o.obj# object_id, o.type# object_type, o.flags flags, t.hashcode hashcode
  from sys.obj$ o, sys.user$ u, sys.type$ t
  where o.owner#=u.user# and o.type#=13 and o.oid$=t.tvoid;


create or replace view sys.cdb$tables&pdbid sharing=none as
select * from sys.cdb$objects&pdbid where object_type=2;

-- object-linked view for accessing sysauth$: common system privileges
create or replace view sys.int$cdb$commonsysprivs&pdbid sharing=object as
select u.name, s.privilege# from sys.user$ u, sys.sysauth$ s
where s.grantee#=u.user# and bitand(s.option$,8)=8 and s.privilege#<0;

create or replace view sys.cdb$commonsysprivs&pdbid as
select * from sys.int$cdb$commonsysprivs&pdbid;

-- object-linked view for accessing sysauth$: common role grants
create or replace view sys.int$cdb$commonrolegrants&pdbid sharing=object as
select u.name, r.name rolename from sys.user$ u, sys.sysauth$ s, sys.user$ r
where s.grantee#=u.user# and bitand(s.option$,8)=8 and s.privilege#>0 and
s.privilege#=r.user#;

create or replace view sys.cdb$commonrolegrants&pdbid as
select * from sys.int$cdb$commonrolegrants&pdbid;

-- object-linked view for accessing objauth$
create or replace view sys.int$cdb$commonobjprivs&pdbid sharing=object as
select u.name, oa.privilege#, u2.name owner, o.name objname, oa.col#
from sys.user$ u, sys.objauth$ oa, sys.obj$ o, sys.user$ u2
where oa.grantee#=u.user# and bitand(oa.option$,8)=8 
and oa.obj#=o.obj# and o.owner#=u2.user#;

create or replace view sys.cdb$commonobjprivs&pdbid as
select * from sys.int$cdb$commonobjprivs&pdbid;


---------------------------------------------------------------------------
-- PRE-SCRIPT CHECKS GO HERE:

-- Check that we have no invalid table data
DOC
#######################################################################
#######################################################################

     The following statement will cause an "ORA-01722: invalid number"
     error, if the database contains invalid data as a result of type
     evolution which was performed without the data being converted.
     
     To resolve this specific "ORA-01722: invalid number" error:
       Perform the data conversion (details below) in the pluggable database.

     Please refer to Oracle Database Object-Relational Developer's Guide
     for more information about type evolution.

     Data in columns of evolved types must be converted before the
     database can be converted.

     The following commands, run inside the PDB, will perform the data
     conversion for Oracle supplied data:

     @?/rdbms/admin/utluppkg.sql
     SET SERVEROUTPUT ON;
     exec dbms_preup.run_fixup_and_report('INVALID_SYS_TABLEDATA');
     SET SERVEROUTPUT OFF;

     You should then confirm that any non-Oracle supplied data is also
     converted.  You should review the data and determine if it needs
     to be converted or removed. 

     To view the data that is affected by type evolution, execute the
     following inside the PDB:

     SELECT rpad(u.name,128) TABLENAME, rpad(o.name,128) OWNER,
       rpad(c.name,128) COLNAME FROM SYS.OBJ$ o, SYS.COL$ c, SYS.COLTYPE$ t,
         SYS.USER$ u
         WHERE o.OBJ# = t.OBJ# AND c.OBJ# = t.OBJ# AND c.COL# = t.COL#
           AND t.INTCOL# = c.INTCOL# AND BITAND(t.FLAGS, 256) = 256
           AND o.OWNER# = u.USER# AND o.OWNER# NOT IN
            (SELECT UNIQUE (d.USER_ID) FROM SYS.DBA_USERS d, SYS.REGISTRY$ r
               WHERE d.USER_ID = r.SCHEMA# and r.NAMESPACE='SERVER');

     Once the data is confirmed, the following commands, run inside the PDB, 
     will convert the data returned by the above query.

     @?/rdbms/admin/utluppkg.sql
     SET SERVEROUTPUT ON;
     exec dbms_preup.run_fixup_and_report('INVALID_USR_TABLEDATA');
     SET SERVEROUTPUT OFF;

     Depending on the amount of data involved, converting the evolved type
     data can take a significant amount of time.

     After this is complete, please rerun noncdb_to_pdb.sql.

#######################################################################
#######################################################################
#

declare
  do_abort boolean := false;
begin
  if dbms_preup.condition_exists ('INVALID_SYS_TABLEDATA') then
    -- dump out the info
    dbms_preup.run_check('INVALID_SYS_TABLEDATA');
    do_abort := TRUE;
  end if;
  if dbms_preup.condition_exists ('INVALID_USR_TABLEDATA') THEN
    -- dump out the info
    dbms_preup.run_check('INVALID_USR_TABLEDATA');
    do_abort := TRUE;
  END IF;
  If do_abort THEN
    dbms_output.put_line ('Invalid table data.');
    dbms_output.put_line ('Non-CDB conversion aborting.');
    dbms_output.put_line ('For instructions, look for ORA-01722 in this script.');
    dbms_output.put_line ('Please resolve these and rerun noncdb_to_pdb.sql.');
    RAISE INVALID_NUMBER;
  end if;
end;
/

-- END PRE-SCRIPT CHECKS
---------------------------------------------------------------------------

-- 18478064:
-- In the case that this is run post-upgrade, we should reenable indexes that
-- were disabled during upgrade. That way, ENABLED$INDEXES created below will
-- accurately reflect . This is necessary because indexes can be dropped
-- and recreated during upgrade, and enabled$indexes.objnum would be outdated.
-- In the non-upgrade case, this will be a no-op, because enabled$indexes
-- wouldn't exist
@@?/rdbms/admin/reenable_indexes.sql

-- record enabled indexes, so that if this script disables them implicitly
-- we can reenable them in the end
set serveroutput off

CREATE TABLE sys.enabled$indexes sharing=none ( schemaname, indexname, objnum )
AS select u.name, o1.name, i.obj# from user$ u, obj$ o1, obj$ o2, ind$ i
    where
        u.user# = o1.owner# and o1.type# = 1 and o1.obj# = i.obj#
       and bitand(i.property, 16)= 16 and bitand(i.flags, 1024)=0
       and i.bo# = o2.obj# and bitand(o2.flags, 2)=0;

-- mark users and roles in our PDB as common if they exist as common in ROOT
-- also compare the type# to ensure that we match users in PDB with users in
-- ROOT (and same for roles)
update sys.user$ a set a.spare1=a.spare1+128 where a.user# in ( 
  select p.user# from sys.cdb$common_users&pdbid r, sys.user$ p 
  where r.name=p.name and r.type#=p.type# and bitand(p.spare1, 128)=0);

-- mark privileges in our PDB as common if they exist as common in ROOT
DECLARE
  cursor c is
    select s.grantee#, s.privilege#
      from sys.sysauth$ s, sys.user$ u, sys.cdb$commonsysprivs&pdbid r
     where u.user#=s.grantee# and u.name=r.name and s.privilege#=r.privilege#
           and bitand(s.option$,8)=0;
BEGIN
  FOR obj in c
  LOOP
    BEGIN
      execute immediate 'update sys.sysauth$ set option$=option$+12+'
                     || 'bitand(option$,3)*15 where grantee#='
                     || obj.grantee# || ' and privilege#=' || obj.privilege#;
    END;
  END LOOP;
  commit;
END;
/

DECLARE
  cursor c is
    select s.grantee#, s.privilege#
      from sys.sysauth$ s, sys.user$ u, sys.cdb$commonrolegrants&pdbid r,
           sys.user$ ru
     where u.user#=s.grantee# and u.name=r.name and r.rolename=ru.name and
           s.privilege#=ru.user# and bitand(s.option$,8)=0;
BEGIN
  FOR obj in c
  LOOP
    BEGIN
      execute immediate 'update sys.sysauth$ set option$=option$+12+'
                     || 'bitand(option$,3)*15 where grantee#='
                     || obj.grantee# || ' and '
                     || ' privilege#=' || obj.privilege#;
    END;
  END LOOP;
  commit;
END;
/

DECLARE
  cursor c is
    select oa.privilege#, oa.grantee#, oa.obj#, oa.col#
      from sys.objauth$ oa, sys.user$ u, sys.cdb$commonobjprivs&pdbid r,
           sys.obj$ o, sys.user$ ou
     where oa.privilege#=r.privilege# and oa.grantee#=u.user# and
           u.name=r.name and o.name=r.objname and oa.obj#=o.obj# and
           ou.name=r.owner and o.owner#=ou.user# and bitand(oa.option$,8)=0 and
           ((oa.col# is null and r.col# is null) or oa.col#=r.col#);

BEGIN
  FOR obj in c
  LOOP
    DECLARE
      colstr VARCHAR(30);
    BEGIN
      IF (obj.col# is null) THEN
        colstr := ' IS NULL';
      ELSE
        colstr := '=' || obj.col#;
      END IF;
      
      execute immediate 'update sys.objauth$ set option$=option$+12+'
                     || 'bitand(option$,3)*15 where grantee#='
                     || obj.grantee# || ' and privilege#=' || obj.privilege#
                     || ' and obj#=' || obj.obj# || ' and col#' || colstr;
    END;
  END LOOP;
  commit;
END;
/


select to_char(sysdate, 'Dy DD-Mon-YYYY HH24:MI:SS') from dual;

-- mark objects in our PDB as common if they exist as common in ROOT
DECLARE
  cursor c is
    select p.object_id, p.flags-bitand(p.flags, 196608) flags,
           decode(r.sharing, 'MDL', 65536, 131072) sharing_flag
      from sys.cdb$common_root_objects&pdbid r, sys.cdb$objects&pdbid p
    where r.owner=p.owner and r.object_name=p.object_name
      and r.object_type=p.object_type and r.object_type != 13 and r.nsp=p.nsp
      and (p.object_subname is null and r.object_subname is null
           or r.object_subname=p.object_subname)
      and decode(bitand(p.flags, 196608), 65536, 'MDL', 131072, 'OBL', 'NONE')<>r.sharing;
  cursor ctyp is
    select p.object_id, p.flags-bitand(p.flags, 196608) flags,
           decode(r.sharing, 'MDL', 65536, 131072) sharing_flag
      from sys.cdb$cmn_root_types&pdbid r, sys.cdb$types&pdbid p
    where r.owner=p.owner and r.object_name=p.object_name
      and (p.object_subname is null and r.object_subname is null
           or r.object_subname=p.object_subname)
      and decode(bitand(p.flags, 196608), 65536, 'MDL', 131072, 'OBL', 'NONE')<>r.sharing
      and p.object_sig=r.object_sig and p.hashcode=r.hashcode;
BEGIN
  FOR obj in c
  LOOP
    BEGIN
      execute immediate 'update sys.obj$ set flags=' || (obj.flags + obj.sharing_flag) ||
                        ' where obj#=' || obj.object_id;
    END;
  END LOOP;
  FOR obj in ctyp
  LOOP
    BEGIN
      execute immediate 'update sys.obj$ set flags=' || (obj.flags + obj.sharing_flag) ||
                        ' where obj#=' || obj.object_id;
    END;
  END LOOP;
  commit;
END;
/

-- Mark types created via xml schema registration as local. Only need to do 
-- this for common types with system generated name.  
DECLARE
  cursor c is
    select p.obj#, p.flags oldflags
    from sys.obj$ p, sys.dependency$ d, sys.obj$ o
    where p.type#=13 and d.p_obj#=p.obj# and d.d_obj#=o.obj# and o.type#=55
          and bitand(p.flags, 196608) != 0
          and ((regexp_instr(p.name, '[0-9]_T',1,1) != 0 and 
                regexp_instr(p.name, '[0-9]_T',1,1) = length(p.name)-2) or 
               (regexp_instr(p.name, '[0-9]_COLL',1,1) != 0 and 
                regexp_instr(p.name, '[0-9]_COLL',1,1) = length(p.name)-4)); 
BEGIN
  FOR obj in c
  LOOP
    BEGIN
      execute immediate 'update sys.obj$ set flags=' || 
                         (obj.oldflags - bitand(obj.oldflags, 196608)) ||
                        ' where obj#=' || obj.obj#;
    END;
  END LOOP;
  commit;
END;
/

-- As types with system generated name created via xml schema registration
-- are marked as local, mark default tables depending on those types as
-- local too, plus default tables with system generated table name.
-- Only need to do this for common tables.  
DECLARE
  cursor c is
  with v as (select p.obj#, p.flags pflags
             from sys.obj$ p, sys.dependency$ d, sys.obj$ o
             where p.type#=13 and p.obj#=d.p_obj# and d.d_obj#=o.obj# 
                   and o.type#=55)
  select distinct o1.obj#, o1.flags oldflags
  from v, sys.dependency$ d1, sys.obj$ o1
  where v.obj#=d1.p_obj# and d1.d_obj#=o1.obj# and o1.type#=2 
        and bitand(o1.flags, 196608) != 0
        and ((bitand(pflags, 196608) = 0) or
             (regexp_instr(o1.name, '[0-9]_TAB',1,1) != 0 and 
              regexp_instr(o1.name, '[0-9]_TAB',1,1) = length(o1.name)-3)); 
BEGIN
  FOR obj in c
  LOOP
    BEGIN
      execute immediate 'update sys.obj$ set flags=' || 
                         (obj.oldflags - bitand(obj.oldflags, 196608)) ||
                        ' where obj#=' || obj.obj#;
    END;
  END LOOP;
  commit;
END;
/

-- As default tables created via xml schema registration are marked as local, 
-- mark all nested tables as local too if its parent table is local.
-- Only need to do this for common nested tables.  
declare 
  -- get local O-R xmltype parent table owned by common users 
  cursor ortabq is 
    select n.owner, n.parent_table_name 
    from sys.obj$ o, dba_users u, dba_nested_tables n, 
    ( select owner, table_name from dba_xml_tables 
      where storage_type='OBJECT-RELATIONAL' 
      union 
      select owner, table_name from dba_xml_tab_cols 
      where storage_type='OBJECT-RELATIONAL' 
     ) t 
    where n.owner = t.owner 
          and n.parent_table_name = t.table_name 
          and t.owner = u.username 
          and u.common='YES' 
          and u.user_id = o.owner# 
          and o.name = t.table_name 
          and bitand(o.flags, 196608) = 0
    order by n.owner, n.parent_table_name; 

  -- get the hierarchy of nested tables, given a parent table name and owner 
  cursor ntq(oname varchar2, pname varchar2) is 
    select n.owner, n.parent_table_name, n.table_name 
    from dba_nested_tables n 
    where n.owner=oname 
    start with parent_table_name=pname 
    connect by prior table_name=parent_table_name 
    order by n.owner, n.parent_table_name, n.table_name;
 
  -- get obj$ entries for the common nested tables, given its owner and name 
  cursor c (owner_name varchar2, obj_name varchar2) is 
    select o.obj#, o.flags oldflags
    from sys.obj$ o, dba_users u
    where o.owner#=u.user_id and u.username=owner_name and o.name=obj_name
          and bitand(o.flags, 196608) != 0;
begin 
  for rec in ortabq loop 
    for ntrec in ntq(rec.owner, rec.parent_table_name) loop
      for obj in c(ntrec.owner, ntrec.table_name) loop
        begin 
          execute immediate 'update sys.obj$ set flags=' || 
                            (obj.oldflags - bitand(obj.oldflags, 196608)) ||
                            ' where obj#=' || obj.obj#;
        end;
      end loop; 
      commit;
    end loop; 
  end loop; 
end; 
/ 

select to_char(sysdate, 'Dy DD-Mon-YYYY HH24:MI:SS') from dual;

alter system flush shared_pool;

-- generate signatures for the common tables which don't have them
DECLARE
  cursor c is
    select r.owner, r.object_name
      from sys.cdb$common_root_objects&pdbid r, sys.cdb$tables&pdbid p
    where r.owner=p.owner and r.object_name=p.object_name
      and r.object_type=2 and p.object_sig is null
      and p.object_name not in ('OBJ$', 'USER$');
BEGIN
  FOR tab in c
  LOOP
    BEGIN
      execute immediate 'ALTER TABLE ' || tab.owner || '."' ||
                        tab.object_name || '" UPGRADE';
    EXCEPTION
      WHEN OTHERS THEN
      BEGIN
        IF (sqlcode = -600 or sqlcode = -602 or sqlcode = -603) THEN
          raise;
        END IF;
      END;
    END;
  END LOOP;
  commit;
END;
/

-- for each table whose signature doesn't match ROOT's, mark its PL/SQL
-- dependents for local MCode 
DECLARE
  cursor c is
    select obj#
      from sys.obj$ o, sys.user$ u, sys.cdb$common_root_objects&pdbid ro
    where o.type# <> 4 and u.name=ro.owner and u.user#=o.owner#
      and o.name=ro.object_name and o.type#=ro.object_type and obj# in
      (select d_obj# from sys.dependency$ where p_obj# in
        (select p.object_id from sys.CDB$common_root_objects&pdbid r,
                                 sys.cdb$tables&pdbid p
         where r.owner=p.owner and r.object_name=p.object_name
           and r.object_type=2 and r.object_sig <> p.object_sig));
BEGIN
  FOR obj in c
  LOOP
    execute immediate
      'update sys.obj$ set flags=flags+33554432-bitand(flags, 33554432) where obj#=' || obj.obj#;
  END LOOP;
  commit;
END;
/

select owner#, name from sys.obj$ where bitand(flags, 33554432)=33554432
  order by 1, 2;

@@?/rdbms/admin/utlip

-- normalize dependencies for MDL java objects (includes classes.bin objects)
delete from sys.dependency$ where d_obj# in (select obj# from sys.obj$ where bitand(flags,65536)=65536 and type# in (29,56));

insert into sys.dependency$ (select do.obj#,do.stime,order#,po.obj#,po.stime,do.owner#,property,d_attrs,d_reason from sys.obj$ do,sys.user$ du,sys.obj$ po,sys.user$ pu,sys.cdb$rootdeps&pdbid rd where du.user#=do.owner# and pu.user#=po.owner# and do.name=rd.name and du.name=owner and do.type#=d_type# and po.name=referenced_name and pu.name=referenced_owner and po.type#=p_type# and bitand(do.flags,65536)=65536 and do.type# in (29,56));

commit;

-- get rid of idl_ub1$ rows for MDL java objects
delete from sys.idl_ub1$ where obj# in (select obj# from sys.obj$ where bitand(flags, 65536)=65536 and type# in (28,29,30,56));
commit;

alter system flush shared_pool;
/

-- explicitly compile these now, before close/reopen. Otherwise they would
-- be used/validated within PDB Open, where such patching (clearing of dict
-- rows) can't be done.
alter public synonym ALL_OBJECTS compile; 
alter view V_$PARAMETER compile;

WHENEVER SQLERROR CONTINUE;
alter type ANYDATA compile;
WHENEVER SQLERROR EXIT;

alter pluggable database "&pdbname" close;

alter pluggable database "&pdbname" open restricted;

alter session set "_ORACLE_SCRIPT"=false;

select status, count(*) from sys.obj$ where status in (5, 6) group by status order by 1;
select count(*) from sys.view$;
select count(*) from sys.obj$ where bitand(flags, 65536)=65536 and obj# in
  (select obj# from sys.view$);
select count(*) from sys.procedure$;
select count(*) from sys.obj$ where bitand(flags, 65536)=65536 and obj# in
  (select obj# from sys.procedure$);
select count(*) from sys.dir$;
select count(*) from sys.obj$ where bitand(flags, 65536)=65536 and obj# in
  (select obj# from sys.dir$);

@@utlrp

select status, count(*) from sys.obj$ where status in (5, 6) group by status order by 1;
select count(*) from sys.view$;
select count(*) from sys.obj$ where bitand(flags, 65536)=65536 and obj# in
  (select obj# from sys.view$);
select count(*) from sys.procedure$;
select count(*) from sys.obj$ where bitand(flags, 65536)=65536 and obj# in
  (select obj# from sys.procedure$);
select count(*) from sys.dir$;
select count(*) from sys.obj$ where bitand(flags, 65536)=65536 and obj# in
  (select obj# from sys.dir$);

-- mark old version types as valid, as utlrp skips these
update sys.obj$ set status = 1
  where type#=13 and subname is not null and status > 1;
commit;

alter pluggable database "&pdbname" close;
alter system flush shared_pool;
/
/
alter pluggable database "&pdbname" open upgrade;


alter session set "_ORACLE_SCRIPT"=true;

drop view sys.cdb$tables&pdbid;
drop view sys.cdb$objects&pdbid;
drop view sys.cdb$types&pdbid;
drop view sys.cdb$common_root_objects&pdbid;
drop view sys.int$cdb$common_root_objects&pdbid;
drop view sys.cdb$cmn_root_types&pdbid;
drop view sys.int$cdb$cmn_root_types&pdbid;
drop view sys.cdb$common_users&pdbid;
drop view sys.int$cdb$common_users&pdbid;
drop view sys.cdb$rootdeps&pdbid;
drop view sys.int$cdb$rootdeps&pdbid;

alter session set container=CDB$ROOT;
drop view sys.cdb$common_root_objects&pdbid;
drop view sys.int$cdb$common_root_objects&pdbid;
drop view sys.cdb$cmn_root_types&pdbid;
drop view sys.int$cdb$cmn_root_types&pdbid;
drop view sys.cdb$common_users&pdbid;
drop view sys.int$cdb$common_users&pdbid;
drop view sys.cdb$rootdeps&pdbid;
drop view sys.int$cdb$rootdeps&pdbid;

alter session set container="&pdbname";

-- handle Resource Manager plan conversions
exec dbms_rmin.rm$_noncdb_to_pdb;

-- delete SYS$BACKGROUND and SYS$USERS from service$
delete from sys.service$ where name in ('SYS$BACKGROUND', 'SYS$USERS');
commit;

-- reset the parameters at the end of the script
exec dbms_pdb.noncdb_to_pdb(2);

-- run ALTER TABLE UPGRADE on table dependents of common types
-- note this is done after noncdb_to_pdb(2), because between (1) and (2),
-- the driver was changed to only compute signatures
DECLARE
  cursor c is
    select u.name owner, o.name object_name
      from sys.obj$ o, sys.user$ u
    where o.type#=2 and u.user#=o.owner# and obj# in
      (select d_obj# from sys.dependency$ d, sys.obj$ typo where
       typo.type#=13 and typo.obj#=d.p_obj# and d.p_timestamp <> typo.stime and
       bitand(typo.flags, 196608)<>0);
BEGIN
  FOR tab in c
  LOOP
    BEGIN
      execute immediate 'ALTER TABLE ' ||
                        dbms_assert.enquote_name(tab.owner, FALSE) || '.' ||
                        dbms_assert.enquote_name(tab.object_name, FALSE) ||
                        ' UPGRADE';
    EXCEPTION
      WHEN OTHERS THEN
      BEGIN
        IF (sqlcode = -600 or sqlcode = -602 or sqlcode = -603) THEN
          raise;
        END IF;
      END;
    END;
  END LOOP;
  commit;
END;
/

alter session set "_ORACLE_SCRIPT"=false;

alter pluggable database "&pdbname" close;
alter session set container = CDB$ROOT;
alter system flush shared_pool;
/
/
alter session set container = "&pdbname";

-- leave the PDB in the same state it was when we started
BEGIN
  execute immediate '&open_sql &restricted_state';
EXCEPTION
  WHEN OTHERS THEN
  BEGIN
    IF (sqlcode <> -900) THEN
      RAISE;
    END IF;
  END;
END;
/


WHENEVER SQLERROR CONTINUE;
